# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-16 12:50
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0006_index_queue_monotonic_position'),
    ]

    operations = [
        migrations.AddField(
            model_name='attendee',
            name='completed_timestamp',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='attendee',
            name='created_timestamp',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='attendee',
            name='updated_timestamp',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
