{% extends "wafer/base.html" %}
{% load i18n %}
{% load crispy_forms_tags %}
{% block content %}
<h1>Volunteers awaiting shirts</h2>
<p>
  The following volunteers have met the minimum requirement for a shirt.
  Check them off when they receive their shirt.
</p>
{% crispy form %}
{% endblock %}
